#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// By TradingCoders.com - Plots opening price of a time during a time period.
    /// </summary>
    [Description("By TradingCoders.com - Plots opening price of a time during a time period.")]
    public class KnCmeClosePriceV2 : Indicator
    {
        #region Variables
        // Wizard generated variables
		
            private int 		startTime 				= 2215; // Default setting for StartTime
            private int 		endTime 				= 2000; // Default setting for EndTime
		
			private bool		showPlot				= true;
			private Color		lineColor				= Color.Black;
			private int 		plot0Width 				= 1;
			private DashStyle	dash0Style				= DashStyle.Custom;
			private PlotStyle	plot0Style				= PlotStyle.TriangleLeft;
		
			private bool		showText				= true;
			private Font 		textFont				= new Font("Arial", 12f, FontStyle.Bold);
			private Color 		textColor				= Color.Black;
			private Color 		outlineColor			= Color.Transparent;
			private Color 		areaColor				= Color.Transparent;
			private int 		zopacity				= 0;
		
        // User defined variables (add any user defined variables below)
		double openPrice = 0;
		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.SteelBlue,2), PlotStyle.TriangleRight, "CmeClosePrice"));
			
            Overlay				= true;
			CalculateOnBarClose = false;
			PlotsConfigurable 	= false;
        }
		
		protected override void OnStartUp()
		{
			Plots[0].PlotStyle = plot0Style;
			Plots[0].Pen.Width = plot0Width;
			Plots[0].Pen.DashStyle = dash0Style;
			
			if(showPlot)
				Plots[0].Pen.Color = lineColor;
			else	
				Plots[0].Pen.Color = Color.Empty;
		}

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < 1)
				return;
			
			// find the open
            if (	(	ToTime(Time[1]) <= startTime*100
					&&	ToTime(Time[0]) >  startTime*100)
				||	(	ToTime(Time[1]) >= startTime*100)
					&&	ToTime(Time[0]) >	startTime*100
					&&	ToDay(Time[1])  <	ToDay(Time[0]))
			{
				if (openPrice == 0)
				{
					openPrice = Open[0];
					if (showText)
					if (ChartControl != null)
						DrawText("price"+CurrentBar,true,Instrument.MasterInstrument.Round2TickSize(openPrice).ToString(),Time[1],openPrice,0,textColor,textFont,StringAlignment.Far,outlineColor,areaColor,Zopacity);
				}
			}
			
			// find the end time
			if (	(	ToTime(Time[1]) <= endTime*100
					&&	ToTime(Time[0]) >  endTime*100)
				||	(	ToTime(Time[1]) >= endTime*100)
					&&	ToTime(Time[0]) >	endTime*100
					&&	ToDay(Time[1])  <	ToDay(Time[0]))
			{
				openPrice = 0;
			}
			
			if (openPrice != 0)
				OpeningPrice.Set(openPrice);
		}

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries OpeningPrice
        {
            get { return Values[0]; }
        }

        [Description("Opening price of the bar starting at or after this time in HHMM")]
        [GridCategory("Parameters")]
        public int StartTime
        {
            get { return startTime; }
            set { startTime = Math.Max(0, value); }
        }

        [Description("End time to cease plotting, in HHMM")]
        [GridCategory("Parameters")]
        public int EndTime
        {
            get { return endTime; }
            set { endTime = Math.Max(0, value); }
        }
		
		[Description("Show Opening Price Line")]
        [Category("Price Line")]
		[Gui.Design.DisplayName ("1.Show Plot")]
        public bool ShowPlot
        {
            get { return showPlot; }
            set { showPlot = value; }
        }
		
		/// <summary>
		/// </summary>
		[Description("Width for Price Line")]
		[Category("Price Line")]
		[Gui.Design.DisplayNameAttribute("3.Line Width")]
		public int Plot0Width
		{
			get { return plot0Width; }
			set { plot0Width = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for Price Line")]
		[Category("Price Line")]
		[Gui.Design.DisplayNameAttribute("4.Dash Style")]
		public DashStyle Dash0Style
		{
			get { return dash0Style; }
			set { dash0Style = value; }
		} 
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for Price Line")]
		[Category("Price Line")]
		[Gui.Design.DisplayNameAttribute("5.Plot Style")]
		public PlotStyle Plot0Style
		{
			get { return plot0Style; }
			set { plot0Style = value; }
		} 
		
		/// <summary>
		/// </summary>
		[Description("Select color for Price Line")]
		[Category("Price Line")]
		[Gui.Design.DisplayName("2.Color Price Line")]
		public Color LineColor
		{
			get { return lineColor; }
			set { lineColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string LineColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(lineColor); }
			set { lineColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[Description("Text marker")]
		[Gui.Design.DisplayName ("1.Show Text")]
        [Category("Text")]
        public bool ShowText
        {
            get { return showText; }
            set { showText = value; }
        }
		
		[XmlIgnore()]
		[Description("Text Font")]
        [Category("Text")]
        [Gui.Design.DisplayNameAttribute("2.Text Font")]
        public Font TextFont
        {
            get { return textFont; }
            set { textFont = value; }
        }
		
		[Browsable(false)]
        public string textFontSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableFont.ToString(textFont); }
            set { textFont = NinjaTrader.Gui.Design.SerializableFont.FromString(value); }
        }
		
		[XmlIgnore]
        [Description("Text Color")]
        [Category("Text")]
        [Gui.Design.DisplayNameAttribute("3.Text Color")]
        public Color TextColor
        {
            get { return textColor; }
            set { textColor = value; }
        }
		
		[Browsable(false)]
        public string textColorSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(textColor); }
            set { textColor = Gui.Design.SerializableColor.FromString(value); }
        }
		
		[XmlIgnore]
        [Description("Text Box Area Color")]
        [Category("Text")]
        [Gui.Design.DisplayNameAttribute("5.Text Box Area Color")]
        public Color AreaColor
        {
            get { return areaColor; }
            set { areaColor = value; }
        }

        [Browsable(false)]
        public string AreaColorSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(areaColor); }
            set { areaColor = Gui.Design.SerializableColor.FromString(value); }
        }        
		
		[XmlIgnore]
        [Description("Text Box Outline Color")]
        [Category("Text")]
        [Gui.Design.DisplayNameAttribute("6.Text Box Outline Color")]
        public Color OutlineColor
        {
            get { return outlineColor; }
            set { outlineColor = value; }
        }

        [Browsable(false)]
        public string OutlineColorSerialize
        {
            get { return Gui.Design.SerializableColor.ToString(outlineColor); }
            set { outlineColor = Gui.Design.SerializableColor.FromString(value); }
        }
		
		[Description("Zone Opacity")]
		[Category("Text")]
		[Gui.Design.DisplayNameAttribute("7.Text Box Opacity")]
		public int Zopacity
		{
			get { return zopacity; }
			set { zopacity = value; }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private KnCmeClosePriceV2[] cacheKnCmeClosePriceV2 = null;

        private static KnCmeClosePriceV2 checkKnCmeClosePriceV2 = new KnCmeClosePriceV2();

        /// <summary>
        /// By TradingCoders.com - Plots opening price of a time during a time period.
        /// </summary>
        /// <returns></returns>
        public KnCmeClosePriceV2 KnCmeClosePriceV2(int endTime, int startTime)
        {
            return KnCmeClosePriceV2(Input, endTime, startTime);
        }

        /// <summary>
        /// By TradingCoders.com - Plots opening price of a time during a time period.
        /// </summary>
        /// <returns></returns>
        public KnCmeClosePriceV2 KnCmeClosePriceV2(Data.IDataSeries input, int endTime, int startTime)
        {
            if (cacheKnCmeClosePriceV2 != null)
                for (int idx = 0; idx < cacheKnCmeClosePriceV2.Length; idx++)
                    if (cacheKnCmeClosePriceV2[idx].EndTime == endTime && cacheKnCmeClosePriceV2[idx].StartTime == startTime && cacheKnCmeClosePriceV2[idx].EqualsInput(input))
                        return cacheKnCmeClosePriceV2[idx];

            lock (checkKnCmeClosePriceV2)
            {
                checkKnCmeClosePriceV2.EndTime = endTime;
                endTime = checkKnCmeClosePriceV2.EndTime;
                checkKnCmeClosePriceV2.StartTime = startTime;
                startTime = checkKnCmeClosePriceV2.StartTime;

                if (cacheKnCmeClosePriceV2 != null)
                    for (int idx = 0; idx < cacheKnCmeClosePriceV2.Length; idx++)
                        if (cacheKnCmeClosePriceV2[idx].EndTime == endTime && cacheKnCmeClosePriceV2[idx].StartTime == startTime && cacheKnCmeClosePriceV2[idx].EqualsInput(input))
                            return cacheKnCmeClosePriceV2[idx];

                KnCmeClosePriceV2 indicator = new KnCmeClosePriceV2();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.EndTime = endTime;
                indicator.StartTime = startTime;
                Indicators.Add(indicator);
                indicator.SetUp();

                KnCmeClosePriceV2[] tmp = new KnCmeClosePriceV2[cacheKnCmeClosePriceV2 == null ? 1 : cacheKnCmeClosePriceV2.Length + 1];
                if (cacheKnCmeClosePriceV2 != null)
                    cacheKnCmeClosePriceV2.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheKnCmeClosePriceV2 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// By TradingCoders.com - Plots opening price of a time during a time period.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.KnCmeClosePriceV2 KnCmeClosePriceV2(int endTime, int startTime)
        {
            return _indicator.KnCmeClosePriceV2(Input, endTime, startTime);
        }

        /// <summary>
        /// By TradingCoders.com - Plots opening price of a time during a time period.
        /// </summary>
        /// <returns></returns>
        public Indicator.KnCmeClosePriceV2 KnCmeClosePriceV2(Data.IDataSeries input, int endTime, int startTime)
        {
            return _indicator.KnCmeClosePriceV2(input, endTime, startTime);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// By TradingCoders.com - Plots opening price of a time during a time period.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.KnCmeClosePriceV2 KnCmeClosePriceV2(int endTime, int startTime)
        {
            return _indicator.KnCmeClosePriceV2(Input, endTime, startTime);
        }

        /// <summary>
        /// By TradingCoders.com - Plots opening price of a time during a time period.
        /// </summary>
        /// <returns></returns>
        public Indicator.KnCmeClosePriceV2 KnCmeClosePriceV2(Data.IDataSeries input, int endTime, int startTime)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.KnCmeClosePriceV2(input, endTime, startTime);
        }
    }
}
#endregion
